@@ -3,7 +3,6 @@ import cv2
 import numpy as np
 
 from typing import Tuple, Union
-from functools import singledispatchmethod
 
 from .constant import Place, SHOW_INDEX
 from .coordinate import Rect, Size
@@ -17,8 +16,7 @@ except AttributeError:
 
 
 class _Image(object):
-    def __init__(self, data: Union[str, bytes, np.ndarray, cv2.cuda.GpuMat, cv2.Mat, cv2.UMat],
-                 read_mode: int = cv2.IMREAD_COLOR, dtype=np.uint8, place=Place.Mat, clone: bool = True):
+    def __init__(self, data, read_mode=cv2.IMREAD_COLOR, dtype=np.uint8, place=Place.Mat, clone=True):
         """
         基础构造函数
 
@@ -44,8 +42,7 @@ class _Image(object):
         if data is not None:
             self.write(data, read_mode=self._read_mode, dtype=self.dtype, place=self._place, clone=clone)
 
-    def write(self, data: Union[str, bytes, np.ndarray, cv2.cuda.GpuMat, cv2.Mat, cv2.UMat],
-              read_mode: int = None, dtype=None, place=None, clone=True):
+    def write(self, data, read_mode=None, dtype=None, place=None, clone=True):
         """
         写入图片数据
 
@@ -99,7 +96,7 @@ class _Image(object):
         # logger.debug(f'输出type={type(self._data)}, id={id(self._data)}, place={place}')
 
     @classmethod
-    def _create_mat(cls, data: Union[np.ndarray, cv2.Mat], shape: Union[tuple, list]):
+    def _create_mat(cls, data, shape):
         if len(shape) == 2:  # 当Mat和Ndarray为单通道时,shape会缺少通道
             shape = shape + (1,)
 
@@ -205,21 +202,21 @@ class _Image(object):
         self._place = place
 
     @property
-    def shape(self) -> Tuple[int, int, int]:
+    def shape(self):
         """
         获取图片的长、宽、通道数
 
         Returns:
             shape: (长,宽,通道数)
         """
-        if self._place in (Place.Mat, Place.Ndarray):
+        if self.place in (Place.Mat, Place.Ndarray):
             shape = self.data.shape
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             shape = self.data.size()[::-1] + (self.data.channels(),)
-        elif self._place == Place.UMat:
+        elif self.place == Place.UMat:
             shape = self.data.get().shape
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
 
         if len(shape) == 2:  # 当Mat和Ndarray为单通道时,shape会缺少通道
             shape = shape + (1,)
@@ -244,11 +241,11 @@ class _Image(object):
         Returns:
             channels: 通道数
         """
-        if self._place in (Place.Mat, Place.Ndarray):
+        if self.place in (Place.Mat, Place.Ndarray):
             return self.shape[2]
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             return self.data.channels()
-        elif self._place == Place.UMat:
+        elif self.place == Place.UMat:
             return self.shape[2]
 
     @property
@@ -293,42 +290,96 @@ class Image(_Image):
         clone = kwargs.pop('clone', True)
         return Image(data=data, read_mode=self._read_mode, dtype=self.dtype, place=self.place, clone=clone)
 
-    @singledispatchmethod
-    def resize(self, w: int, h: int, code: int = cv2.INTER_LINEAR):
+    def rotate(self, code):
         """
-        调整图片大小
+        旋转图片
 
         Args:
-            w(int): 需要设定的宽
-            h(int): 需要设定的长
-            code(int): 缩放方法 https://docs.opencv.org/4.x/da/d54/group__imgproc__transform.html#ga5bb5a1fea74ea38e1a5445ca803ff121
+            code: 可选90度180度270度(顺时针)
 
         Returns:
-            Image: 调整大小后的图像
+            Image: 旋转后的图片
         """
-        if self._place == Place.Mat:
-            data = cv2.resize(self.data, (int(w), int(h)), interpolation=code)  # return: np.ndarray
-            data = self._create_mat(data, data.shape)
-        elif self._place in (Place.Ndarray, Place.UMat):
-            data = cv2.resize(self.data, (int(w), int(h)), interpolation=code)
-        elif self._place == Place.GpuMat:
-            data = cv2.cuda.resize(self.data, (int(w), int(h)), interpolation=code)
+        assert code in (cv2.ROTATE_180, cv2.ROTATE_90_COUNTERCLOCKWISE, cv2.ROTATE_90_CLOCKWISE)
+
+        if self.place in (Place.Mat, Place.Ndarray, Place.UMat):
+            data = cv2.rotate(self.data, code)
+        elif self.place == Place.GpuMat:
+            if code == cv2.ROTATE_180:
+                size = self.size[::-1]
+                angle = 180
+                offset_x, offset_y = size
+            elif code == cv2.ROTATE_90_CLOCKWISE:
+                size = self.size
+                angle = 90
+                offset_y = size[1]
+                offset_x = 0
+            else:
+                size = self.size
+                angle = 270
+                offset_y = 0
+                offset_x = size[0]
+            data = cv2.cuda.rotate(self.data, size, angle, xShift=offset_x, yShift=offset_y)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
         return self._clone_with_params(data, clone=False)
 
-    @resize.register(Size)
-    def _(self, size: Size, code: int = cv2.INTER_LINEAR):
-        """
-        调整图片大小
+    def resize(self, *args, **kwargs):
+        code = kwargs.get('code', cv2.INTER_LINEAR)
 
-        Args:
-            size: 需要设置的长宽
+        if kwargs.get('size'):
+            size = kwargs.get('size')
+            if isinstance(size, Size):  # args: (size=Size(100, 100), code=2)
+                w, h = size.width, size.height
+            elif isinstance(size, (tuple, list)):  # args: (size=(100, 100), code=2)
+                w, h = size
+            else:
+                raise ValueError('Unknown params args={}, kwargs={}'.format(args, kwargs))
+        elif kwargs.get('w') and kwargs.get('h'):  # args: (w=100, h=100, code=2)
+            w, h = kwargs.get('w'), kwargs.get('h')
+        else:
+            args_len = len(args)
+            if args_len in (1, 2):
+                if args_len == 1:  # args:(Size(100, 100)) or ((100, 100))
+                    arg = args[0]
+                else:
+                    if isinstance(args[0], (Size, tuple, list)):  # args:(Size(100, 100),2) or ((100, 100), 2)
+                        arg, code = args
+                    else:  # args:(100, 100) or Size(100, 100)
+                        arg = args
 
-        Returns:
-            Image: 调整大小后的图像
-        """
-        return self.resize(int(size.width), int(size.height), code=code)
+                if isinstance(arg, Size):
+                    w, h = arg.width, arg.height
+                elif isinstance(arg, (tuple, list)):
+                    if len(arg) == 2:
+                        w, h = arg
+                    else:
+                        raise ValueError('Unknown params args={}, kwargs={}'.format(args, kwargs))
+                else:
+                    raise ValueError('Unknown params args={}, kwargs={}'.format(args, kwargs))
+            elif args_len == 3:
+                w, h, code = args
+            else:
+                raise ValueError('Unknown params args={}, kwargs={}'.format(args, kwargs))
+
+        assert type(w) == int, '参数必须是int类型, args={}, kwargs={}'.format(args, kwargs)
+        assert type(h) == int, '参数必须是int类型 args={}, kwargs={}'.format(args, kwargs)
+        assert type(code) == int, '参数必须是int类型 args={}, kwargs={}'.format(args, kwargs)
+
+        return self._resize(w=w, h=h, code=code)
+
+    def _resize(self, w, h, code=cv2.INTER_LINEAR):
+        size = (w, h)
+        if self.place == Place.Mat:
+            data = cv2.resize(self.data, size, interpolation=code)  # return: np.ndarray
+            data = self._create_mat(data, data.shape)
+        elif self.place in (Place.Ndarray, Place.UMat):
+            data = cv2.resize(self.data, size, interpolation=code)
+        elif self.place == Place.GpuMat:
+            data = cv2.cuda.resize(self.data, size, interpolation=code)
+        else:
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
+        return self._clone_with_params(data, clone=False)
 
     def cvtColor(self, code):
         """
@@ -341,15 +392,15 @@ class Image(_Image):
         Returns:
             Image: 转换后的新图片
         """
-        if self._place == Place.Mat:
+        if self.place == Place.Mat:
             data = cv2.cvtColor(self.data, code)  # return np.ndarray
             data = self._create_mat(data, shape=data.shape)
-        elif self._place in (Place.Ndarray, Place.UMat):
+        elif self.place in (Place.Ndarray, Place.UMat):
             data = cv2.cvtColor(self.data, code)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             data = cv2.cuda.cvtColor(self.data, code)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
 
         return self._clone_with_params(data, clone=False)
 
@@ -367,20 +418,20 @@ class Image(_Image):
         if not Rect(0, 0, width, height).contains(rect):
             raise OverflowError('Rect不能超出屏幕 rect={}, tl={}, br={}'.format(rect, rect.tl, rect.br))
 
-        if self._place in (Place.Mat, Place.Ndarray):
+        if self.place in (Place.Mat, Place.Ndarray):
             x_min, y_min = int(rect.tl.x), int(rect.tl.y)
             x_max, y_max = int(rect.br.x), int(rect.br.y)
             data = self.data.copy()[y_min:y_max, x_min:x_max]
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             data = cv2.cuda.GpuMat(self.data, rect.totuple())
-        elif self._place == Place.UMat:
+        elif self.place == Place.UMat:
             data = cv2.UMat(self.data, rect.totuple())
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
 
         return self._clone_with_params(data, clone=False)
 
-    def threshold(self, thresh: int = 0, maxval: int = 255, code=cv2.THRESH_OTSU):
+    def threshold(self, thresh=0, maxval=255, code=cv2.THRESH_OTSU):
         """
         图片二值化
 
@@ -392,22 +443,22 @@ class Image(_Image):
         Returns:
              Image: 二值化后的图片
         """
-        if self._place == Place.Mat:
+        if self.place == Place.Mat:
             _, data = cv2.threshold(self.data, thresh, maxval, code)  # return: np.ndarray
             data = self._create_mat(data=data, shape=data.shape)
-        elif self._place in (Place.Ndarray, Place.UMat):
+        elif self.place in (Place.Ndarray, Place.UMat):
             _, data = cv2.threshold(self.data, thresh, maxval, code)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             if code > 4:
                 # cuda threshold不支持这两种方法,需要转换
                 _, data = cv2.threshold(self.data.download(), thresh, maxval, code)
             else:
                 _, data = cv2.cuda.threshold(self.data, thresh, maxval, code)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
         return self._clone_with_params(data, clone=False)
 
-    def rectangle(self, rect: Rect, color: Tuple[int, int, int] = (0, 255, 0), thickness: int = 1, lineType=cv2.LINE_8) -> None:
+    def rectangle(self, rect, color=(0, 255, 0), thickness=1, lineType=cv2.LINE_8):
         """
         在图像上画出矩形, 注!绘制会在原图上进行,不会产生新的图片对象
 
@@ -423,15 +474,37 @@ class Image(_Image):
         pt1 = rect.tl
         pt2 = rect.br
 
-        if self._place in (Place.Mat, Place.Ndarray, Place.UMat):
+        if self.place in (Place.Mat, Place.Ndarray, Place.UMat):
             cv2.rectangle(self.data, (pt1.x, pt1.y), (pt2.x, pt2.y), color=color, thickness=thickness, lineType=lineType)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             data = cv2.rectangle(self.data.download(), (pt1.x, pt1.y), (pt2.x, pt2.y), color=color, thickness=thickness, lineType=lineType)
             self.data.upload(data)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
 
-    def gaussianBlur(self, size: Tuple[int, int] = (11, 11), sigma: Union[int, float] = 1.5, borderType: int = cv2.BORDER_DEFAULT):
+    def copyMakeBorder(self, top, bottom, left, right, borderType):
+        """
+        扩充边缘
+
+        Args:
+            top(int): 上扩充大小
+            bottom(int): 下扩充大小
+            left(int): 左扩充大小
+            right(int): 右扩充大小
+            borderType(int): 边缘扩充类型
+
+        Returns:
+
+        """
+        if self.place in (Place.Mat, Place.Ndarray, Place.UMat):
+            data = cv2.copyMakeBorder(self.data, top, bottom, left, right, borderType)
+        elif self.place == Place.GpuMat:
+            data = cv2.cuda.copyMakeBorder(self.data, top, bottom, left, right, borderType)
+        else:
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
+        return self._clone_with_params(data, clone=False)
+
+    def gaussianBlur(self, size=(11, 11), sigma=1.5, borderType=cv2.BORDER_DEFAULT):
         """
         使用高斯滤镜模糊图像
 
@@ -445,19 +518,19 @@ class Image(_Image):
         if not (size[0] % 2 == 1) or not (size[1] % 2 == 1):
             raise ValueError('Window size must be odd.')
 
-        if self._place == Place.Mat:
+        if self.place == Place.Mat:
             data = cv2.GaussianBlur(self.data, ksize=size, sigmaX=sigma, sigmaY=sigma, borderType=borderType)
             data = self._create_mat(data=data, shape=data.shape)
-        elif self._place in (Place.Ndarray, Place.UMat):
+        elif self.place in (Place.Ndarray, Place.UMat):
             data = cv2.GaussianBlur(self.data, ksize=size, sigmaX=sigma, sigmaY=sigma, borderType=borderType)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             dtype = self.data.type()
             # TODO: 感觉可以优化
             gaussian = cv2.cuda.createGaussianFilter(dtype, dtype, ksize=size, sigma1=sigma, sigma2=sigma,
                                                      rowBorderMode=borderType, columnBorderMode=borderType)
             data = gaussian.apply(self.data)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
         return self._clone_with_params(data, clone=False)
 
     def bitwise_not(self, mask=None):
@@ -470,16 +543,16 @@ class Image(_Image):
         Returns:
              Image: 反转后的图片
         """
-        if self._place in (Place.Mat, Place.Ndarray, Place.UMat):
+        if self.place in (Place.Mat, Place.Ndarray, Place.UMat):
             data = cv2.bitwise_not(self.data, mask=mask)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             data = cv2.cuda.bitwise_not(self.data, mask=mask)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
 
         return self._clone_with_params(data, clone=False)
 
-    def imshow(self, title: str = None, flag: int = cv2.WINDOW_KEEPRATIO) -> None:
+    def imshow(self, title=None, flag=cv2.WINDOW_KEEPRATIO):
         """
         以GUI显示图片
 
@@ -502,23 +575,21 @@ class Image(_Image):
         elif isinstance(data, cv2.cuda.GpuMat):
             cv2.imshow(title, data.download())
 
-    def imwrite(self, fileName: str) -> None:
+    def imwrite(self, file_name):
         """
         讲图片保存到指定路径
 
         Args:
-            fileName: 文件路径
-            write_mode: 写入模式
-                https://docs.opencv.org/4.x/d8/d6a/group__imgcodecs__flags.html#ga292d81be8d76901bff7988d18d2b42ac
+            file_name: 文件路径
 
         Returns:
             None
         """
         data = self.data
         if isinstance(data, (np.ndarray, cv2.UMat)):
-            cv2.imwrite(fileName, data)
+            cv2.imwrite(file_name, data)
         elif isinstance(data, cv2.cuda.GpuMat):
-            cv2.imwrite(fileName, data.download())
+            cv2.imwrite(file_name, data.download())
 
     def split(self):
         """
@@ -527,10 +598,10 @@ class Image(_Image):
         Returns:
             拆分后的图像数据,不会对数据包装处理
         """
-        if self._place in (Place.Mat, Place.Ndarray, Place.UMat):
+        if self.place in (Place.Mat, Place.Ndarray, Place.UMat):
             data = cv2.split(self.data)
-        elif self._place == Place.GpuMat:
+        elif self.place == Place.GpuMat:
             data = cv2.cuda.split(self.data)
         else:
-            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self._place, self.data, type(self.data)))
+            raise TypeError("Unknown place:'{}', image_data={}, image_data_type".format(self.place, self.data, type(self.data)))
         return data
